/* The FreeDOS-32 FAT Driver version 2.0
 * Copyright (C) 2001-2005  Salvatore ISAJA
 *
 * This file "file.c" is part of the FreeDOS-32 FAT Driver (the Program).
 *
 * The Program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The Program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Program; see the file GPL.txt; if not, write to
 * the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * \file
 * \brief Facilities to access open files.
 */
/**
 * \addtogroup fat
 * @{
 */
#include "fat.h"

/**
 * \brief  Backend for the "lseek" POSIX system call.
 * \param  c      the file instance to seek into;
 * \param  offset new byte offset for the file pointer according to \c whence;
 * \param  whence can be \c SEEK_SET, \c SEEK_CUR or \c SEEK_END; the latter
 *                is not allowed for directories.
 * \return On success, the new byte offset from the beginning of the file,
 *         or a negative error.
 */
off_t fat_lseek(Channel *c, off_t offset, int whence)
{
	off_t res = -EINVAL;
	if (!c) return -EFAULT;
	if (c->magic != FAT_CHANNEL_MAGIC) return -EBADF;
	switch (whence)
	{
		case SEEK_SET: res = offset; break;
		case SEEK_CUR: res = offset + c->file_pointer; break;
		case SEEK_END:
			if (!(c->f->de.attr & FAT_ADIR))
				res = offset + c->f->de.file_size;
			break;
	}
	if (res < 0) return -EINVAL;
	c->file_pointer = res;
	return res;
}


/**
 * \brief  Backend for the "read" POSIX system call.
 * \param  c      the file instance to read from;
 * \param  buffer pointer to a buffer to receive the data;
 * \param  size   the number of bytes to read;
 * \return The number of bytes read on success (may be less than \c size, 0 at EOF), or a negative error.
 */
ssize_t fat_read(Channel *c, void *buffer, size_t size)
{
	off_t    offset;
	size_t   k = 0, count;
	unsigned byte_in_sector;
	int      res;
	Sector   sector, sector_index;
	Buffer  *b = NULL;
	File    *f = c->f;
	Volume  *v = f->v;

	if (!c || !buffer) return -EFAULT;
	if (c->magic != FAT_CHANNEL_MAGIC) return -EBADF;
	assert(c->file_pointer >= 0);
	if (IS_NOT_READABLE(c)) return -EBADF;
	offset = c->file_pointer;
    //printf("size to read = %d\n", size);
	while (k < size)
	{
		if (!(f->de.attr & FAT_ADIR) && (offset >= f->de.file_size)) break; /* EOF */
		/* Locate the current sector and byte in sector position */
		sector_index = offset >> v->log_bytes_per_sector;
		res = fat_get_file_sector(c, sector_index, &sector);
		if (res < 0) {
            printf("4. Failed to get file sector: %d\n", sector);
            return res;
        }
		if (res > 0) break; /* EOF */
		byte_in_sector = offset & (v->bytes_per_sector - 1);

		/* Read as much as we can in that sector with a single operation.
		 * However, don't read past EOF or more than "size" bytes. */
		count = v->bytes_per_sector - byte_in_sector;
		if (!(f->de.attr & FAT_ADIR) && (offset + count >= f->de.file_size))
			count = f->de.file_size - offset;
		if (k + count > size)
			count = size - k;

		/* Fetch the sector, read data and continue */
		res = fat_readbuf(v, sector, &b, false);
		if (res < 0) {
            printf("5. Failed to read sector: %d\n", sector);
            return res;
        }
		memcpy((uint8_t *) buffer + k, b->data + res + byte_in_sector, count);
		offset += count;
		k      += count;
       // printf("k = %d\n", k);
	}
	/* Successful exit, update file last-access time-stamp if required */
	c->file_pointer = offset;
	return (ssize_t) k;
}

/* @} */
