-------------------------------------------------------------------------------
--
-- VGA Controller
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity vga_controller is
  
  port (
    reset 		: in std_logic;
    clk   		: in std_logic;                    -- Should be 25.125 MHz
    read       	: in  std_logic;
    write      	: in  std_logic;
    chipselect 	: in  std_logic;
	waitrequest : out std_logic;
    address    	: in  unsigned(7 downto 0);
    readdata   	: out unsigned(31 downto 0);
    writedata  	: in  unsigned(31 downto 0); --doubled to 32bit bus
	
    VGA_CLK,                         -- Clock
    VGA_HS,                          -- H_SYNC
    VGA_VS,                          -- V_SYNC
    VGA_BLANK,                       -- BLANK
    VGA_SYNC 	: out std_logic;        -- SYNC
    VGA_R,                           -- Red[9:0]
    VGA_G,                           -- Green[9:0]
    VGA_B 		: out std_logic_vector (9 downto 0); -- Blue[9:0]

	--  Sram Module :
	SRAM_DQ   			 : inout unsigned(15 downto 0);
	SRAM_ADDR            : out unsigned(17 downto 0);
	SRAM_UB_N, SRAM_LB_N : out std_logic;
	SRAM_WE_N, SRAM_CE_N : out std_logic;
	SRAM_OE_N            : out std_logic;
	SRAM_OCCUPY					 : out std_logic
    );

end vga_controller;

architecture rtl of vga_controller is

  -- video constants
	constant HTOTAL       			: integer := 800;
	constant HSYNC        			: integer := 96;
	constant HBACK_PORCH  			: integer := 48;
	constant HACTIVE      			: integer := 640;
	constant HFRONT_PORCH 			: integer := 16;
	constant HACTIVE_CNT			: integer := HSYNC + HBACK_PORCH;
  
	constant VTOTAL       			: integer := 525;
	constant VSYNC        			: integer := 2;
	constant VBACK_PORCH  			: integer := 33;
	constant VACTIVE      			: integer := 480;
	constant VFRONT_PORCH 			: integer := 10;
	constant VACTIVE_CNT			: integer := VSYNC + VBACK_PORCH;

	-- sprite constants
	constant N_SPRITE_ANIMATION		: integer := 16;
	constant N_SPRITE_CURSOR		: integer := 1;
	constant N_SPRITE_TOTAL			: integer := N_SPRITE_ANIMATION + N_SPRITE_CURSOR;
	constant SPRITE_ANIMATION_WIDTH	: integer := 128;
	constant SPRITE_ANIMATION_HEIGHT: integer := 64;
	constant SPRITE_CURSOR_WIDTH	: integer := 9;
	constant SPRITE_CURSOR_HEIGHT	: integer := 12;
	constant Border_VStart			: integer := 336;
	constant Border_Width			: integer := 5;
	
	-- screen constants
	constant MARGIN_LEFTMOST		: integer := 0;
	constant MARGIN_RIGHTMOST		: integer := 0;
	constant MARGIN_UPPEREND		: integer := 0;
	constant MARGIN_LOWEREND		: integer := 0;
	
	-- color constants
	constant COLOR_BIT_ALPHA		: integer := 15;
	constant COLOR_R_START			: integer := 10;
	constant COLOR_R_END			: integer := 14;
	constant COLOR_G_START			: integer := 5;
	constant COLOR_G_END			: integer := 9;
	constant COLOR_B_START			: integer := 0;
	constant COLOR_B_END			: integer := 4;

	-- color data
	signal cursor_color_out				: unsigned( 15 downto 0 ) := X"0000";
	signal color_out					: unsigned( 15 downto 0 ) := X"0000";
	signal is_fading_out			: std_logic := '0';
	signal is_gading_in				: std_logic := '0';
	signal current_frame			: integer := 0;
	
	-- cursor sprite
	signal is_cursor_drawing 		: std_logic := '0';
	signal cursor_pos_x				: unsigned( 9 downto 0 ) := "0001000000";		-- 0 ~ 640
	signal cursor_pos_y				: unsigned( 9 downto 0 ) := "0011000000";		-- 0 ~ 480

	---------------------------------------------------------------------------------------------
	
  type HStartMatrix is array (0 to 3) of unsigned(9 downto 0); 
  signal Card_HStart : HStartMatrix := ("0000000000","0010101010","0101010100","0111111110");
  type VStartMatrix is array (0 to 3) of integer; 
  signal Card_VStart : VStartMatrix := ( 45, 154, 263, 372 );

  -- Signals for the video controller
  signal Hcount : unsigned(9 downto 0);  -- Horizontal position (0-800)
  signal Vcount : unsigned(9 downto 0);  -- Vertical position (0-524) 
  signal EndOfLine, EndOfField : std_logic;
  signal vga_hblank, vga_hsync, vga_vblank, vga_vsync : std_logic;  -- Sync. signals

  signal pic0 : std_logic := '0'; signal pic1 : std_logic := '0'; signal pic2 : std_logic := '0'; signal pic3 : std_logic := '0';
  signal pic4 : std_logic := '0'; signal pic5 : std_logic := '0'; signal pic6 : std_logic := '0'; signal pic7 : std_logic := '0';
  signal pic8 : std_logic := '0'; signal pic9 : std_logic := '0'; signal pic10 : std_logic := '0'; signal pic11 : std_logic := '0';
  signal pic12 : std_logic := '0'; signal pic13 : std_logic := '0'; signal pic14 : std_logic := '0'; signal pic15 : std_logic := '0';
  signal border : std_logic := '0';
  signal clk_25 : std_logic := '0';

	signal sprite_status	 : unsigned(31 downto 0) := X"00000000";
	
	-- animation status
	type TYPE_ANIMATION_STATUS is array( 0 to 15 ) of unsigned( 31 downto 0 );
	signal animation_status : TYPE_ANIMATION_STATUS := ( others => X"00000000" );

-- sram signals
signal ram_address		: unsigned( 17 downto 0 ) 		:= "00" & X"0000";
signal ram_data				: unsigned( 15 downto 0 ) 		:= X"0000";
signal ram_rw_status	: std_logic 									:= '0';


begin
  --Feeding the Sram control signals
  ram_data <= SRAM_DQ;
  SRAM_ADDR <= ram_address;
  SRAM_WE_N <= '1'; 
	SRAM_OE_N <= '0';
  SRAM_UB_N <= '0';
	SRAM_LB_N <= '0';
	SRAM_CE_N <= '0';
	SRAM_OCCUPY <= not ram_rw_status;

-- Halving the clock that is fed to this component since VGA supports 25Mhz instead of 50Mhz  
  process (clk)
  begin
    if rising_edge(clk) then
      clk_25 <= not clk_25;
    end if;
  end process;

RECV_SPRITE_ANIMATION : process(clk)
begin
	if rising_edge(clk) then
		if chipselect = '1' and write = '1' then
			case address is
				when X"04" => animation_status( 0 ) <= writedata;
				when X"05" => animation_status( 1 ) <= writedata;
				when X"06" => animation_status( 2 ) <= writedata;
				when X"07" => animation_status( 3 ) <= writedata;
				when X"08" => animation_status( 4 ) <= writedata;
				when X"09" => animation_status( 5 ) <= writedata;
				when X"0A" => animation_status( 6 ) <= writedata;
				when X"0B" => animation_status( 7 ) <= writedata;
				when X"0C" => animation_status( 8 ) <= writedata;
				when X"0D" => animation_status( 9 ) <= writedata;
				when X"0E" => animation_status( 10 ) <= writedata;
				when X"0F" => animation_status( 11 ) <= writedata;
				when X"10" => animation_status( 12 ) <= writedata;
				when X"11" => animation_status( 13 ) <= writedata;
				when X"12" => animation_status( 14 ) <= writedata;
				when X"13" => animation_status( 15 ) <= writedata;
				when others => animation_status( 15 ) <= animation_status( 15 );
			end case;
		end if;
	end if;
end process RECV_SPRITE_ANIMATION;

RECV_SRAM_STATUS : process (clk)
  begin 
	if rising_edge(clk) then
		if chipselect = '1' then
			if write = '1' then
				if address = X"02" then -- address reserved for sram r/w
					ram_rw_status <= std_logic( writedata( 0 ) );
				end if;
			end if;
		end if;
	end if;
end process RECV_SRAM_STATUS;

RECV_SPRITE_STATUS : process (clk)
begin 
	if rising_edge(clk) then
		if chipselect = '1' then
			if write = '1' then
				if address = X"03" then --address reserved for clicked image array
					sprite_status <= writedata(31 downto 0);
				end if;
			end if;
		end if;
	end if;
end process RECV_SPRITE_STATUS;

-- Getting the X and Y positions for cursor from Nios2 and writing those coordinates into 2 predefined registers
-- namely cursor_pos_x and cursor_pos_y. Assuming the first 10 bits have cursor_pos_X data and 
-- the second 10 bits are holding cursor_pos_Y data
  GetCursorPos : process (clk)
  begin
	if rising_edge(clk) then
		if chipselect = '1' then
			if write = '1' then
				if address = X"00" then
					cursor_pos_x <= writedata(9 downto 0);
					cursor_pos_y <= writedata(19 downto 10);
				end if;
			end if;
 		end if;
	end if;
  end process GetCursorPos;

  -- Horizontal and vertical counters
  HCounter : process (clk_25)
  begin
    if rising_edge(clk_25) then      
      if reset = '1' then
        Hcount <= (others => '0');
      elsif EndOfLine = '1' then
        Hcount <= (others => '0');
      else
        Hcount <= Hcount + 1;
      end if;      
    end if;
  end process HCounter;

  EndOfLine <= '1' when Hcount = HTOTAL - 1 else '0';
  
  VCounter: process (clk_25)
  begin
    if rising_edge(clk_25) then      
      if reset = '1' then
        Vcount <= (others => '0');
      elsif EndOfLine = '1' then
        if EndOfField = '1' then
          Vcount <= (others => '0');
        else
          Vcount <= Vcount + 1;
        end if;
      end if;
    end if;
  end process VCounter;

  EndOfField <= '1' when Vcount = VTOTAL - 1 else '0';

  -- State machines to generate HSYNC, VSYNC, HBLANK, and VBLANK

  HSyncGen : process (clk_25)
  begin
    if rising_edge(clk_25) then     
      if reset = '1' or EndOfLine = '1' then
        vga_hsync <= '1';
      elsif Hcount = HSYNC - 1 then
        vga_hsync <= '0';
      end if;
    end if;
  end process HSyncGen;
  
  HBlankGen : process (clk_25)
  begin
    if rising_edge(clk_25) then
      if reset = '1' then
        vga_hblank <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH then
        vga_hblank <= '0';
      elsif Hcount = HSYNC + HBACK_PORCH + HACTIVE then
        vga_hblank <= '1';
      end if;      
    end if;
  end process HBlankGen;

  VSyncGen : process (clk_25)
  begin
    if rising_edge(clk_25) then
      if reset = '1' then
        vga_vsync <= '1';
      elsif EndOfLine ='1' then
        if EndOfField = '1' then
          vga_vsync <= '1';
        elsif Vcount = VSYNC - 1 then
          vga_vsync <= '0';
        end if;
      end if;      
    end if;
  end process VSyncGen;

  VBlankGen : process (clk_25)
  begin
    if rising_edge(clk_25) then    
      if reset = '1' then
        vga_vblank <= '1';
      elsif EndOfLine = '1' then
        if Vcount = VSYNC + VBACK_PORCH - 1 then
          vga_vblank <= '0';
        elsif Vcount = VSYNC + VBACK_PORCH + VACTIVE - 1 then
          vga_vblank <= '1';
        end if;
      end if;
    end if;
  end process VBlankGen;

	-- Cursor Sprite Generation
	CursorGen : process (clk_25)
	constant cur_inner_color : unsigned( 15 downto 0 ) := X"C86C";
	type TYPE_CURSOR_H_COLOR is array( 0 to ( SPRITE_CURSOR_WIDTH - 1 ) ) of unsigned( 15 downto 0 );
	variable cursor_h_color_0 : TYPE_CURSOR_H_COLOR := ( X"FFFF", X"0000", X"0000", X"0000", X"0000", X"0000", X"0000", X"0000", X"0000" );
	variable cursor_h_color_1 : TYPE_CURSOR_H_COLOR := ( X"FFFF", X"FFFF", X"0000", X"0000", X"0000", X"0000", X"0000", X"0000", X"0000" );
	variable cursor_h_color_2 : TYPE_CURSOR_H_COLOR := ( X"FFFF", cur_inner_color, X"FFFF", X"0000", X"0000", X"0000", X"0000", X"0000", X"0000" );
	variable cursor_h_color_3 : TYPE_CURSOR_H_COLOR := ( X"FFFF", cur_inner_color, cur_inner_color, X"FFFF", X"0000", X"0000", X"0000", X"0000", X"0000" );
	variable cursor_h_color_4 : TYPE_CURSOR_H_COLOR := ( X"FFFF", cur_inner_color, cur_inner_color, cur_inner_color, X"FFFF", X"0000", X"0000", X"0000", X"0000" );
	variable cursor_h_color_5 : TYPE_CURSOR_H_COLOR := ( X"FFFF", cur_inner_color, cur_inner_color, cur_inner_color, cur_inner_color, X"FFFF", X"0000", X"0000", X"0000" );
	variable cursor_h_color_6 : TYPE_CURSOR_H_COLOR := ( X"FFFF", cur_inner_color, cur_inner_color, cur_inner_color, cur_inner_color, cur_inner_color, X"FFFF", X"0000", X"0000" );
	variable cursor_h_color_7 : TYPE_CURSOR_H_COLOR := ( X"FFFF", X"FFFF", X"FFFF", cur_inner_color, X"FFFF", X"FFFF", X"FFFF", X"0000", X"0000" );
	variable cursor_h_color_8 : TYPE_CURSOR_H_COLOR := ( X"0000", X"0000", X"FFFF", cur_inner_color, X"FFFF", X"0000", X"0000", X"0000", X"0000" );
	variable cursor_h_color_9 : TYPE_CURSOR_H_COLOR := ( X"0000", X"0000", X"0000", X"FFFF", cur_inner_color, X"FFFF", X"0000", X"0000", X"0000" );
	variable cursor_h_color_10 : TYPE_CURSOR_H_COLOR := ( X"0000", X"0000", X"0000", X"FFFF", cur_inner_color, X"FFFF", X"0000", X"0000", X"0000" );
	variable cursor_h_color_11 : TYPE_CURSOR_H_COLOR := ( X"0000", X"0000", X"0000", X"0000", X"FFFF", X"FFFF", X"0000", X"0000", X"0000" );
	
	variable cursor_h_color_cur : TYPE_CURSOR_H_COLOR := ( others => X"0000" );
	variable cursor_pixel_color_cur : unsigned( 15 downto 0 ) := X"0000";
	
	begin
	if rising_edge(clk_25) then
		if reset = '1' then
				is_cursor_drawing <= '0';
		-- check whether the current horizontal pixel is pointing cursor area
		elsif ( HACTIVE_CNT + cursor_pos_x - 1 < Hcount and Hcount < HACTIVE_CNT + cursor_pos_x + SPRITE_CURSOR_WIDTH ) and
				( VACTIVE_CNT + cursor_pos_y - 1 < Vcount and Vcount < VACTIVE_CNT + cursor_pos_y + SPRITE_CURSOR_HEIGHT ) then

			-- get current cursor line
			case( to_integer( Vcount - VACTIVE_CNT - cursor_pos_y ) ) is
				when 0 => cursor_h_color_cur := cursor_h_color_0;
				when 1 => cursor_h_color_cur := cursor_h_color_1;
				when 2 => cursor_h_color_cur := cursor_h_color_2;
				when 3 => cursor_h_color_cur := cursor_h_color_3;
				when 4 => cursor_h_color_cur := cursor_h_color_4;
				when 5 => cursor_h_color_cur := cursor_h_color_5;
				when 6 => cursor_h_color_cur := cursor_h_color_6;
				when 7 => cursor_h_color_cur := cursor_h_color_7;
				when 8 => cursor_h_color_cur := cursor_h_color_8;
				when 9 => cursor_h_color_cur := cursor_h_color_9;
				when 10 => cursor_h_color_cur := cursor_h_color_10;
				when 11 => cursor_h_color_cur := cursor_h_color_11;
				when others => cursor_h_color_cur := ( others => X"0000" );
			end case;
			
			-- get current pixel
			cursor_pixel_color_cur := cursor_h_color_cur( to_integer( Hcount - HACTIVE_CNT - cursor_pos_x ) );
			
			-- set alpha value and color
			is_cursor_drawing <= cursor_pixel_color_cur( COLOR_BIT_ALPHA );
			cursor_color_out <= cursor_pixel_color_cur;
		else
			is_cursor_drawing <= '0';
		end if;     
	end if;
  end process CursorGen;

  -- Sprite Generation
	SpriteGen : process (clk_25)
	type TYPE_OFFSET_TABLE is array( integer range 0 to 29, integer range 0 to 127 ) of integer;
	constant offset_table_squizing : TYPE_OFFSET_TABLE :=
(
(   2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2 ),
( 255, 255, 255,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   4,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   4,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   4,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   4,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   4,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255,   2,   2,   2,   2,    2,   2,   2,   2,   4,   2,   2,   2,   2,   2,    2,   2,   2,   4,   2,   2,   2,   2,   2,   2,    2,   2,   4,   2,   2,   2,   2,   2,   2,   2,    2,   4,   2,   2,   2,   2,   2,   2,   2,   2,    4,   2,   2,   2,   2,   2,   2,   2,   2,   4,    2,   2,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   2,   2,   2,   2,   2,   2,   4,   2,    2,   2,   2,   2,   2,   2,   2,   4,   2,   2,    2,   2,   2,   2,   2,   2,   4,   2,   2,   2,    2,   2,   2,   2,   2,   4,   2,   2,   2,   2,    2,   2,   2,   2,   4,   2,   2,   2,   2,   2,    2,   2, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255,   2,    2,   2,   2,   2,   2,   2,   4,   2,   2,   2,    2,   4,   2,   2,   2,   2,   4,   2,   2,   2,    2,   4,   2,   2,   2,   2,   2,   2,   2,   2,    2,   4,   2,   2,   2,   2,   4,   2,   2,   2,    2,   4,   2,   2,   2,   2,   4,   2,   2,   2,    2,   4,   2,   2,   2,   2,   2,   2,   2,   2,    2,   2,   4,   2,   2,   2,   2,   4,   2,   2,    2,   2,   4,   2,   2,   2,   2,   4,   2,   2,    2,   2,   2,   2,   2,   2,   2,   4,   2,   2,    2,   2,   4,   2,   2,   2,   2,   4,   2,   2,    2,   2,   4,   2,   2,   2,   2,   2,   2, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255,   2,   2,   2,   2,   2,   2,   4,   2,    2,   2,   4,   2,   2,   2,   4,   2,   2,   2,    4,   2,   2,   2,   4,   2,   2,   2,   4,   2,    2,   2,   4,   2,   2,   2,   4,   2,   2,   2,    4,   2,   2,   2,   4,   2,   2,   2,   4,   2,    2,   2,   4,   2,   2,   2,   2,   2,   2,   2,    4,   2,   2,   2,   4,   2,   2,   2,   4,   2,    2,   2,   4,   2,   2,   2,   4,   2,   2,   2,    4,   2,   2,   2,   4,   2,   2,   2,   4,   2,    2,   2,   4,   2,   2,   2,   4,   2,   2,   2,    4,   2,   2,   2,   2,   2, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255,   2,   2,   2,   2,   2,    4,   2,   2,   4,   2,   2,   4,   2,   2,   4,    2,   2,   4,   2,   2,   4,   2,   2,   4,   2,    2,   4,   2,   2,   4,   2,   2,   4,   2,   2,    4,   2,   2,   4,   2,   2,   4,   2,   2,   2,    2,   4,   2,   2,   2,   2,   2,   4,   2,   4,    2,   2,   4,   2,   2,   4,   2,   2,   4,   2,    2,   4,   2,   2,   4,   2,   2,   4,   2,   2,    4,   2,   2,   4,   2,   2,   4,   2,   2,   4,    2,   2,   4,   2,   2,   4,   2,   2,   4,   2,    2,   2,   2, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255,   2,   2,    2,   2,   4,   2,   4,   2,   2,   4,   2,   4,    2,   2,   4,   2,   4,   2,   2,   4,   2,   4,    2,   2,   4,   2,   4,   2,   2,   4,   2,   4,    2,   2,   4,   2,   4,   2,   4,   2,   2,   4,    2,   4,   2,   2,   2,   4,   2,   4,   2,   4,    2,   2,   4,   2,   4,   2,   4,   2,   2,   4,    2,   4,   2,   2,   4,   2,   4,   2,   2,   4,    2,   4,   2,   2,   4,   2,   4,   2,   2,   4,    2,   4,   2,   2,   4,   2,   4,   2,   2,   2,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255,   2,   2,   4,   2,   4,   2,   4,   2,   4,    2,   4,   2,   4,   2,   4,   2,   4,   2,   4,    2,   4,   2,   4,   2,   4,   2,   4,   2,   4,    2,   4,   2,   4,   2,   4,   2,   4,   2,   4,    2,   4,   2,   2,   2,   4,   2,   4,   2,   4,    2,   4,   2,   4,   2,   4,   2,   4,   2,   4,    2,   4,   2,   4,   2,   4,   2,   4,   2,   4,    2,   4,   2,   4,   2,   4,   2,   4,   2,   4,    2,   4,   2,   4,   2,   4,   2, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255,   2,   4,   4,   2,   4,   2,    4,   4,   2,   4,   2,   4,   4,   2,   4,   4,    2,   4,   2,   4,   4,   2,   4,   2,   4,   4,    2,   4,   2,   4,   4,   2,   4,   2,   4,   4,    2,   4,   2,   2,   4,   2,   2,   4,   2,   4,    4,   2,   4,   2,   4,   4,   2,   4,   2,   4,    4,   2,   4,   2,   4,   4,   2,   4,   2,   4,    4,   2,   4,   4,   2,   4,   2,   4,   4,   2,    4,   2,   4,   4, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255,   2,   4,   4,   4,    2,   4,   4,   2,   4,   4,   2,   4,   4,   2,    4,   4,   2,   4,   4,   2,   4,   4,   2,   4,    4,   2,   4,   4,   2,   4,   4,   2,   4,   4,    2,   4,   4,   2,   4,   2,   4,   4,   2,   4,    4,   2,   4,   4,   2,   4,   4,   2,   4,   4,    2,   4,   4,   2,   4,   4,   2,   4,   4,   2,    4,   4,   2,   4,   4,   2,   4,   4,   2,   4,    4,   4, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),

( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255,   2,   4,    4,   4,   2,   4,   4,   4,   4,   2,   4,   4,    4,   2,   4,   4,   4,   4,   2,   4,   4,   4,    2,   4,   4,   4,   4,   2,   4,   4,   4,   2,    4,   4,   2,   4,   4,   4,   2,   4,   4,   2,    4,   4,   4,   2,   4,   4,   4,   4,   2,   4,    4,   4,   2,   4,   4,   4,   4,   2,   4,   4,    4,   2,   4,   4,   4,   4,   2,   4,   4,   4,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,    2,   4,   4,   4,   4,   4,   2,   4,   4,   4,    4,   4,   4,   2,   4,   4,   4,   4,   4,   4,    2,   4,   4,   4,   4,   4,   4,   2,   4,   4,    4,   4,   4,   4,   4,   4,   4,   4,   4,   4,    4,   2,   4,   4,   4,   4,   4,   4,   2,   4,    4,   4,   4,   4,   4,   2,   4,   4,   4,   4,    4,   4,   2,   4,   4,   4,   4,   4, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255,   2,   4,   4,   4,   4,   4,   4,   4,    4,   4,   4,   4,   4,   4,   4,   4,   4,   4,    4,   4,   4,   4,   4,   4,   4,   4,   4,   4,    4,   4,   4,   4,   4,   4,   4,   4,   4,   4,    4,   4,   4,   4,   4,   4,   4,   4,   4,   4,    4,   4,   4,   4,   4,   4,   4,   4,   4,   4,    4,   4,   4,   4,   4,   4, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255,   2,   4,   4,   4,   4,   4,    4,   4,   6,   4,   4,   4,   4,   4,   4,   6,    4,   4,   4,   4,   4,   6,   4,   4,   4,   4,    4,   6,   4,   4,   4,   4,   4,   6,   4,   4,    4,   4,   4,   6,   4,   4,   4,   4,   4,   6,    4,   4,   4,   4,   4,   4,   6,   4,   4,   4,    4,   4,   4,   4, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255,   2,   4,   4,   6,    6,   4,   6,   4,   6,   4,   4,   4,   6,   4,    4,   4,   6,   4,   4,   4,   6,   4,   4,   4,    6,   4,   4,   4,   4,   4,   4,   4,   6,   4,    4,   4,   6,   4,   4,   4,   6,   4,   4,   4,    6,   4,   4,   4,   6,   4,   6,   4,   6,   6,    4,   4, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255,   2,   4,    4,   6,   6,   4,   6,   4,   6,   4,   6,   4,    6,   4,   6,   4,   6,   4,   6,   4,   6,   4,    6,   4,   6,   4,   4,   4,   6,   4,   6,   4,    6,   4,   6,   4,   6,   4,   6,   4,   6,   4,    6,   4,   6,   4,   6,   4,   6,   6,   4,   4,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,    2,   4,   4,   6,   6,   6,   4,   6,   6,   6,    4,   6,   6,   6,   4,   6,   6,   6,   4,   6,    6,   6,   4,   6,   4,   6,   4,   6,   6,   6,    4,   6,   6,   6,   4,   6,   6,   6,   4,   6,    6,   6,   4,   6,   6,   6,   4,   4, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255,   2,   4,   4,   6,   6,   6,   6,   6,    6,   6,   6,   6,   6,   6,   6,   6,   6,   6,    6,   6,   6,   6,   8,   6,   6,   6,   6,   6,    6,   6,   6,   6,   6,   6,   6,   6,   6,   6,    6,   6,   6,   6,   4,   4, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255,   2,   4,   4,   8,   6,   6,    8,   6,   6,   8,   6,   6,   8,   6,   6,   8,    6,   6,   8,   6,   8,   6,   8,   6,   6,   8,    6,   6,   8,   6,   6,   8,   6,   6,   8,   6,    6,   8,   4,   4, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255,   2,   8,   8,   8,    6,   6,   8,   8,   8,   6,   6,   8,   8,   8,    6,   6,   8,   6,   8,   6,   8,   6,   6,   8,    8,   8,   6,   6,   8,   8,   8,   6,   6,   8,    8,   8, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),

( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255,   2,   8,    8,   8,   8,   8,   8,   8,   8,   8,   8,   8,    8,   8,   8,  10,   8,  10,   8,   8,   8,   8,    8,   8,   8,   8,   8,   8,   8,   8,   8,   8,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,    2,  10,  10,   8,  10,  10,   8,  10,  10,   8,   10,  10,   8,  10,   8,  10,   8,  10,  10,   8,   10,  10,   8,  10,  10,   8,  10,  10, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255,   2,  12,  10,  12,  10,  12,  10,  12,   10,  12,  10,  10,  12,  10,  10,  12,  10,  12,   10,  12,  10,  12,  10,  12, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255,   2,  14,  14,  12,  14,  14,   12,  14,  14,  12,  12,  12,  14,  14,  12,  14,   14,  12,  14,  14, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255,   2,  16,  18,  16,   18,  16,  18,  16,  16,  16,  18,  16,  18,  16,   18,  16, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255,   2,  22,   24,  22,  24,  22,  24,  22,  24,  22,  24,  22,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,    2,  36,  36,  36,  36,  36,  36,  36, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255,   2,  84,  84,  84, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255,   2, 252, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 ),
( 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  255, 255, 255, 255, 255, 255, 255, 255 )
);


	variable color_c : integer := 0; -- signed( 31 downto 0 ) := X"00000000";
	variable color_d : integer := 0; -- signed( 31 downto 0 ) := X"00000000";
	variable color_e : integer := 0; -- signed( 31 downto 0 ) := X"00000000";
	variable color_r_faded : signed( 31 downto 0 ) := X"00000000";
	variable color_g_faded : signed( 31 downto 0 ) := X"00000000";
	variable color_b_faded : signed( 31 downto 0 ) := X"00000000";
	type TYPE_FRAME is array( 0 to 29 ) of integer;
	variable frame : TYPE_FRAME := ( others => 0 );
	begin
		if falling_edge( clk_25 ) then
			if reset = '1' then -- reset
				is_fading_out <= '0';
			else
				-- draw sprite 0
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 0 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 0 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 0 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 0 ) is
						when X"00000000" => -- normal showing
							frame( 0 ) := 0;
							if sprite_status( 16 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
									ram_address <= ( "00" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								else
									pic0 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 0 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 0 ) < 59 then
								frame( 0 ) := frame( 0 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								ram_address <= ( "00" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
							else
								pic0 <= '1';
								if frame( 0 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 0 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 0 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 0 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 0 ) < 59 then
								frame( 0 ) := frame( 0 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								ram_address <= ( "00" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
							else
								pic0 <= '1';
								if frame( 0 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 0 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 0 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 0 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								ram_address <= ( "00" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
							else
								pic0 <= '1';
								ram_address <= ram_address + 2;
								current_frame <= frame( 0 );
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 0 ) < 59 then
								frame( 0 ) := frame( 0 ) + 1;
							end if;
						when others =>
							pic0 <= '0';
					end case;
				-- out of the sprite area
				else
					pic0 <= '0';
				end if;
				
				-- draw sprite 1
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 0 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 1 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 0 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 1 ) is
						when X"00000000" => -- normal showing
							frame( 1 ) := 0;
							if sprite_status( 17 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
									--if sprite_status( 1 ) = '0' then
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
									--else
										ram_address <= ( "00" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
									--end if;
								else
									pic1 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 1 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 1 ) < 59 then
								frame( 1 ) := frame( 1 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								--if frame( 1 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--else
									ram_address <= ( "00" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--end if;
							else
								pic1 <= '1';
								if frame( 1 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 1 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 1 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 1 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 1 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 1 ) < 59 then
								frame( 1 ) := frame( 1 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								--if frame( 1 ) < 30 then
									ram_address <= ( "00" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--end if;
							else
								pic1 <= '1';
								if frame( 1 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 1 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 1 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 1 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 1 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 1 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								ram_address <= ( "00" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
							else
								pic1 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 1 ) < 59 then
								frame( 1 ) := frame( 1 ) + 1;
							end if;
						when others =>
							pic1 <= '0';
					end case;
				-- out of the sprite area
				else
					pic1 <= '0';
				end if;
	
				-- draw sprite 2
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 0 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 2 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 0 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 2 ) is
						when X"00000000" => -- normal showing
							frame( 2 ) := 0;
							if sprite_status( 18 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
									--if sprite_status( 2 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "00" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
									--end if;
								else
									pic2 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 2 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 2 ) < 59 then
								frame( 2 ) := frame( 2 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								--if frame( 2 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--else
									ram_address <= ( "00" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--end if;
							else
								pic2 <= '1';
								if frame( 2 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 2 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 2 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 2 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 2 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 2 ) < 59 then
								frame( 2 ) := frame( 2 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								--if frame( 2 ) < 30 then
									ram_address <= ( "00" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--end if;
							else
								pic2 <= '1';
								if frame( 2 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 2 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 2 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 2 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 2 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 2 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								ram_address <= ( "00" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
							else
								pic2 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 2 ) < 59 then
								frame( 2 ) := frame( 2 ) + 1;
							end if;
						when others =>
							pic2 <= '0';
					end case;
				-- out of the sprite area
				else
					pic2 <= '0';
				end if;

				-- draw sprite 3
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 0 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 3 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 0 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 3 ) is
						when X"00000000" => -- normal showing
							frame( 3 ) := 0;
							if sprite_status( 19 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
									--if sprite_status( 3 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "00" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
									--end if;
								else
									pic3 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 3 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 3 ) < 59 then
								frame( 3 ) := frame( 3 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								--if frame( 3 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--else
									ram_address <= ( "00" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--end if;
							else
								pic3 <= '1';
								if frame( 3 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 3 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 3 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 3 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 3 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 3 ) < 59 then
								frame( 3 ) := frame( 3 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								--if frame( 3 ) < 30 then
									ram_address <= ( "00" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
								--end if;
							else
								pic3 <= '1';
								if frame( 3 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 3 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 3 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 3 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 3 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 3 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								ram_address <= ( "00" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 0 ) ) * 256 );
							else
								pic3 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 0 ) and frame( 3 ) < 59 then
								frame( 3 ) := frame( 3 ) + 1;
							end if;
						when others =>
							pic3 <= '0';
					end case;
				-- out of the sprite area
				else
					pic3 <= '0';
				end if;

				-- draw sprite 4
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 1 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 0 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 1 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 4 ) is
						when X"00000000" => -- normal showing
							frame( 4 ) := 0;
							if sprite_status( 20 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
									--if sprite_status( 4 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "01" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
									--end if;
								else
									pic4 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 4 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 4 ) < 59 then
								frame( 4 ) := frame( 4 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								--if frame( 4 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--else
									ram_address <= ( "01" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--end if;
							else
								pic4 <= '1';
								if frame( 4 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 4 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 4 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 4 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 4 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 4 ) < 59 then
								frame( 4 ) := frame( 4 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								--if frame( 4 ) < 30 then
									ram_address <= ( "01" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--end if;
							else
								pic4 <= '1';
								if frame( 4 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 4 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 4 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 4 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 4 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 4 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								ram_address <= ( "01" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
							else
								pic4 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 4 ) < 59 then
								frame( 4 ) := frame( 4 ) + 1;
							end if;
						when others =>
							pic4 <= '0';
					end case;
				-- out of the sprite area
				else
					pic4 <= '0';
				end if;
				
				-- draw sprite 5
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 1 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 1 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 1 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 5 ) is
						when X"00000000" => -- normal showing
							frame( 5 ) := 0;
							if sprite_status( 21 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
									--if sprite_status( 5 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "01" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
									--end if;
								else
									pic5 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 5 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 5 ) < 59 then
								frame( 5 ) := frame( 5 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								--if frame( 5 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--else
									ram_address <= ( "01" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--end if;
							else
								pic5 <= '1';
								if frame( 5 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 5 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 5 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 5 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 5 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 5 ) < 59 then
								frame( 5 ) := frame( 5 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								--if frame( 5 ) < 30 then
									ram_address <= ( "01" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--end if;
							else
								pic5 <= '1';
								if frame( 5 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 5 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 5 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 5 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 5 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 5 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								ram_address <= ( "01" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
							else
								pic5 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 5 ) < 59 then
								frame( 5 ) := frame( 5 ) + 1;
							end if;
						when others =>
							pic5 <= '0';
					end case;
				-- out of the sprite area
				else
					pic5 <= '0';
				end if;
	
				-- draw sprite 6
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 1 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 2 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 1 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 6 ) is
						when X"00000000" => -- normal showing
							frame( 6 ) := 0;
							if sprite_status( 22 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
									--if sprite_status( 6 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "01" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
									--end if;
								else
									pic6 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 6 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 6 ) < 59 then
								frame( 6 ) := frame( 6 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								--if frame( 6 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--else
									ram_address <= ( "01" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--end if;
							else
								pic6 <= '1';
								if frame( 6 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 6 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 6 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 6 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 6 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 6 ) < 59 then
								frame( 6 ) := frame( 6 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								--if frame( 6 ) < 30 then
									ram_address <= ( "01" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--end if;
							else
								pic6 <= '1';
								if frame( 6 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 6 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 6 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 6 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 6 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 6 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								ram_address <= ( "01" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
							else
								pic6 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 6 ) < 59 then
								frame( 6 ) := frame( 6 ) + 1;
							end if;
						when others =>
							pic6 <= '0';
					end case;
				-- out of the sprite area
				else
					pic6 <= '0';
				end if;
				
				-- draw sprite 7
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 1 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 3 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 1 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 7 ) is
						when X"00000000" => -- normal showing
							frame( 7 ) := 0;
							if sprite_status( 23 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
									--if sprite_status( 7 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "01" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
									--end if;
								else
									pic7 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 7 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 7 ) < 59 then
								frame( 7 ) := frame( 7 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								--if frame( 7 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--else
									ram_address <= ( "01" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--end if;
							else
								pic7 <= '1';
								if frame( 7 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 7 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 7 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 7 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 7 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 7 ) < 59 then
								frame( 7 ) := frame( 7 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								--if frame( 7 ) < 30 then
									ram_address <= ( "01" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
								--end if;
							else
								pic7 <= '1';
								if frame( 7 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 7 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 7 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 7 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 7 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 7 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								ram_address <= ( "01" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 1 ) ) * 256 );
							else
								pic7 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 1 ) and frame( 7 ) < 59 then
								frame( 7 ) := frame( 7 ) + 1;
							end if;
						when others =>
							pic7 <= '0';
					end case;
				-- out of the sprite area
				else
					pic7 <= '0';
				end if;

				-- draw sprite 8
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 2 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 0 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 2 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 8 ) is
						when X"00000000" => -- normal showing
							frame( 8 ) := 0;
							if sprite_status( 24 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
									--if sprite_status( 8 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "10" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
									--end if;
								else
									pic8 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 8 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 8 ) < 59 then
								frame( 8 ) := frame( 8 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								--if frame( 8 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--else
									ram_address <= ( "10" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--end if;
							else
								pic8 <= '1';
								if frame( 8 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 8 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 8 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 8 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 8 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 8 ) < 59 then
								frame( 8 ) := frame( 8 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								--if frame( 8 ) < 30 then
									ram_address <= ( "10" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--end if;
							else
								pic8 <= '1';
								if frame( 8 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 8 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 8 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 8 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 8 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 8 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								ram_address <= ( "10" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
							else
								pic8 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 8 ) < 59 then
								frame( 8 ) := frame( 8 ) + 1;
							end if;
						when others =>
							pic8 <= '0';
					end case;
				-- out of the sprite area
				else
					pic8 <= '0';
				end if;
				
				-- draw sprite 9
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 2 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 1 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 2 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 9 ) is
						when X"00000000" => -- normal showing
							frame( 9 ) := 0;
							if sprite_status( 25 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
									--if sprite_status( 9 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "10" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
									--end if;
								else
									pic9 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 9 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 9 ) < 59 then
								frame( 9 ) := frame( 9 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								--if frame( 9 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--else
									ram_address <= ( "10" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--end if;
							else
								pic9 <= '1';
								if frame( 9 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 9 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 9 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 9 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 9 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 9 ) < 59 then
								frame( 9 ) := frame( 9 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								--if frame( 9 ) < 30 then
									ram_address <= ( "10" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--end if;
							else
								pic9 <= '1';
								if frame( 9 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 9 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 9 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 9 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 9 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 9 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								ram_address <= ( "10" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
							else
								pic9 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 9 ) < 59 then
								frame( 9 ) := frame( 9 ) + 1;
							end if;
						when others =>
							pic9 <= '0';
					end case;
				-- out of the sprite area
				else
					pic9 <= '0';
				end if;

				-- draw sprite 10
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 2 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 2 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 2 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 10 ) is
						when X"00000000" => -- normal showing
							frame( 10 ) := 0;
							if sprite_status( 26 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
									--if sprite_status( 10 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "10" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
									--end if;
								else
									pic10 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 10 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 10 ) < 59 then
								frame( 10 ) := frame( 10 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								--if frame( 10 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--else
									ram_address <= ( "10" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--end if;
							else
								pic10 <= '1';
								if frame( 10 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 10 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 10 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 10 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 10 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 10 ) < 59 then
								frame( 10 ) := frame( 10 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								--if frame( 10 ) < 30 then
									ram_address <= ( "10" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--end if;
							else
								pic10 <= '1';
								if frame( 10 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 10 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 10 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 10 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 10 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 10 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								ram_address <= ( "10" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
							else
								pic10 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 10 ) < 59 then
								frame( 10 ) := frame( 10 ) + 1;
							end if;
						when others =>
							pic10 <= '0';
					end case;
				-- out of the sprite area
				else
					pic10 <= '0';
				end if;
				
				-- draw sprite 11
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 2 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 3 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 2 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 11 ) is
						when X"00000000" => -- normal showing
							frame( 11 ) := 0;
							if sprite_status( 27 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
									--if sprite_status( 11 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "10" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
									--end if;
								else
									pic11 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 11 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 11 ) < 59 then
								frame( 11 ) := frame( 11 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								--if frame( 11 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--else
									ram_address <= ( "10" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--end if;
							else
								pic11 <= '1';
								if frame( 11 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 11 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 11 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 11 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 11 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 11 ) < 59 then
								frame( 11 ) := frame( 11 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								--if frame( 11 ) < 30 then
									ram_address <= ( "10" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
								--end if;
							else
								pic11 <= '1';
								if frame( 11 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 11 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 11 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 11 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 11 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 11 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								ram_address <= ( "10" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 2 ) ) * 256 );
							else
								pic11 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 2 ) and frame( 11 ) < 59 then
								frame( 11 ) := frame( 11 ) + 1;
							end if;
						when others =>
							pic11 <= '0';
					end case;
				-- out of the sprite area
				else
					pic11 <= '0';
				end if;

				-- draw sprite 12
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 3 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 0 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 3 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 12 ) is
						when X"00000000" => -- normal showing
							frame( 12 ) := 0;
							if sprite_status( 28 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
									--if sprite_status( 12 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "11" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
									--end if;
								else
									pic12 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 12 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 12 ) < 59 then
								frame( 12 ) := frame( 12 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								--if frame( 12 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--else
									ram_address <= ( "11" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--end if;
							else
								pic12 <= '1';
								if frame( 12 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 12 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 12 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 12 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 12 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 12 ) < 59 then
								frame( 12 ) := frame( 12 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								--if frame( 12 ) < 30 then
									ram_address <= ( "11" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--end if;
							else
								pic12 <= '1';
								if frame( 12 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 12 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 12 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 12 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 12 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 0 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 12 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 then
								ram_address <= ( "11" & X"0000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
							else
								pic12 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 0 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 12 ) < 59 then
								frame( 12 ) := frame( 12 ) + 1;
							end if;
						when others =>
							pic12 <= '0';
					end case;
				-- out of the sprite area
				else
					pic12 <= '0';
				end if;
				
				-- draw sprite 13
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 3 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 1 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 3 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 13 ) is
						when X"00000000" => -- normal showing
							frame( 13 ) := 0;
							if sprite_status( 29 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
									--if sprite_status( 13 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "11" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
									--end if;
								else
									pic13 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 13 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 13 ) < 59 then
								frame( 13 ) := frame( 13 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								--if frame( 13 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--else
									ram_address <= ( "11" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--end if;
							else
								pic13 <= '1';
								if frame( 13 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 13 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 13 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 13 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 13 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 13 ) < 59 then
								frame( 13 ) := frame( 13 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								--if frame( 13 ) < 30 then
									ram_address <= ( "11" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--end if;
							else
								pic13 <= '1';
								if frame( 13 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 13 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 13 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 13 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 13 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 1 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 13 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 then
								ram_address <= ( "11" & X"4000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
							else
								pic13 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 1 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 13 ) < 59 then
								frame( 13 ) := frame( 13 ) + 1;
							end if;
						when others =>
							pic13 <= '0';
					end case;
				-- out of the sprite area
				else
					pic13 <= '0';
				end if;
	
				-- draw sprite 14
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 3 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 2 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 3 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 14 ) is
						when X"00000000" => -- normal showing
							frame( 14 ) := 0;
							if sprite_status( 30 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
									--if sprite_status( 14 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "11" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
									--end if;
								else
									pic14 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 14 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 14 ) < 59 then
								frame( 14 ) := frame( 14 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								--if frame( 14 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--else
									ram_address <= ( "11" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--end if;
							else
								pic14 <= '1';
								if frame( 14 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 14 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 14 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 14 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 14 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 14 ) < 59 then
								frame( 14 ) := frame( 14 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								--if frame( 14 ) < 30 then
									ram_address <= ( "11" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--else
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--end if;
							else
								pic14 <= '1';
								if frame( 14 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 14 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 14 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 14 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 14 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 2 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 14 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 then
								ram_address <= ( "11" & X"8000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
							else
								pic14 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 2 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 14 ) < 59 then
								frame( 14 ) := frame( 14 ) + 1;
							end if;
						when others =>
							pic14 <= '0';
					end case;
				-- out of the sprite area
				else
					pic14 <= '0';
				end if;
				
				-- draw sprite 15
				-- within this sprite area and when it is activated
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount > VSYNC + VBACK_PORCH + Card_VStart( 3 ) - 1
					and Hcount < HSYNC + HBACK_PORCH + Card_HStart( 3 ) + 128 and Vcount < VSYNC + VBACK_PORCH + Card_VStart( 3 ) + 64 then
					is_fading_out <= '0';
					case animation_status( 15 ) is
						when X"00000000" => -- normal showing
							frame( 15 ) := 0;
							if sprite_status( 31 ) = '0' then	-- if enabled
								if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
									--if sprite_status( 15 ) = '0' then -- show back
										--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
									--else	-- show front
										ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
									--end if;
								else
									pic15 <= '1';
									ram_address <= ram_address + 2;
								end if;
								if sprite_status( 15 ) = '0' then -- show back
									color_out <= X"39DF";
								else
									color_out <= ram_data;
								end if;
							end if;
						when X"00000001" => -- squizing back image and expanding front image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 15 ) < 59 then
								frame( 15 ) := frame( 15 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								--if frame( 15 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--else
									ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--end if;
							else
								pic15 <= '1';
								if frame( 15 ) < 30 then
									-- squize back image
									if offset_table_squizing( frame( 15 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 15 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= X"39DF";
									end if;
								else
									-- expand front image							
									if offset_table_squizing( 59 - frame( 15 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 15 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= ram_data;
									end if;
								end if;
							end if;
						when X"00000002" => -- squizing front image and expanding back image
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 15 ) < 59 then
								frame( 15 ) := frame( 15 ) + 1;
							end if;
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								--if frame( 15 ) < 30 then
									--ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--else
									ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
								--end if;
							else
								pic15 <= '1';
								if frame( 15 ) < 30 then
									-- squize front image
									if offset_table_squizing( frame( 15 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( frame( 15 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= ram_data;
									end if;
								else
									-- expand back image							
									if offset_table_squizing( 59 - frame( 15 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) ) = 255 then
										color_out <= X"0000";
									else
										ram_address <= ram_address + offset_table_squizing( 59 - frame( 15 ), to_integer( Hcount - HSYNC - HBACK_PORCH - Card_HStart( 3 ) ) );
										color_out <= X"39DF";
									end if;
								end if;
							end if;
						when  X"00000003" => -- fading out
							is_fading_out <= '1';
							current_frame <= frame( 15 );
							-- access ram
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 then
								ram_address <= ( "11" & X"C000" ) + to_integer( ( Vcount - VSYNC - VBACK_PORCH - Card_VStart( 3 ) ) * 256 );
							else
								pic15 <= '1';
								ram_address <= ram_address + 2;
							end if;
							-- increase frame
							if Hcount = HSYNC + HBACK_PORCH + Card_HStart( 3 ) - 1 and Vcount = VSYNC + VBACK_PORCH + Card_VStart( 3 ) and frame( 15 ) < 59 then
								frame( 15 ) := frame( 15 ) + 1;
							end if;
						when others =>
							pic15 <= '0';
					end case;
				-- out of the sprite area
				else
					pic15 <= '0';
				end if;
			
				if Hcount >= HSYNC + HBACK_PORCH + Card_HStart( 3 ) + 128 then
					is_fading_out <= '0';
				end if;
			
			end if;
		end if;
  end process SpriteGen;


  -- Registered video signals going to the video DAC
  VideoOut: process (clk_25, reset)
	-- variables for transferring from cursor_ to VGA_R,G,B
	variable color_r : unsigned( 9 downto 0 ) := "0000000000";
	variable color_g : unsigned( 9 downto 0 ) := "0000000000";
	variable color_b : unsigned( 9 downto 0 ) := "0000000000";
	
	variable sprite_active : std_logic := '0';
	
	variable color_c : integer := 0; -- signed( 31 downto 0 ) := X"00000000";
	variable color_d : integer := 0; -- signed( 31 downto 0 ) := X"00000000";
	variable color_e : integer := 0; -- signed( 31 downto 0 ) := X"00000000";
	variable color_r_faded : unsigned( 31 downto 0 ) := X"00000000";
	variable color_g_faded : unsigned( 31 downto 0 ) := X"00000000";
	variable color_b_faded : unsigned( 31 downto 0 ) := X"00000000";
	
  begin
		-- split 15-bit color data into 5-bit R, G, B data
		color_r( 9 downto 5 ) := color_out( 14 downto 10 );
		color_g( 9 downto 5 ) := color_out( 9 downto 5 );
		color_b( 9 downto 5 ) := color_out( 4 downto 0 );

		sprite_active := pic0 or pic1 or pic2 or pic3 or pic4 or pic5 or pic6 or pic7
										or pic8 or pic9 or pic10 or pic11 or pic12 or pic13 or pic14 or pic15;

    if reset = '1' then
      VGA_R <= "0000000000";
      VGA_G <= "0000000000";
      VGA_B <= "0000000000";
    elsif rising_edge( clk_25 ) then
			if is_cursor_drawing = '1' then
				color_r( 9 downto 5 ) := cursor_color_out( COLOR_R_END downto COLOR_R_START );
				color_g( 9 downto 5 ) := cursor_color_out( COLOR_G_END downto COLOR_G_START );
				color_b( 9 downto 5 ) := cursor_color_out( COLOR_B_END downto COLOR_B_START );
				VGA_R <= std_logic_vector( color_r );
        VGA_G <= std_logic_vector( color_g );
        VGA_B <= std_logic_vector( color_b );
      elsif sprite_active = '1' then
				if is_fading_out = '1' then
					-- convert rgb -> cde
					color_c := ( 66 * to_integer( ram_data( 14 downto 10 ) ) + 129 * to_integer( ram_data( 9 downto 5 ) ) + 25 * to_integer( ram_data( 4 downto 0 ) ) );
					color_d := ( -38 * to_integer( ram_data( 14 downto 10 ) ) - 74 * to_integer( ram_data( 9 downto 5 ) ) + 112 * to_integer( ram_data( 4 downto 0 ) ) );
					color_e := ( 112 * to_integer( ram_data( 14 downto 10 ) ) - 94 * to_integer( ram_data( 9 downto 5 ) ) - 18 * to_integer( ram_data( 4 downto 0 ) ) );
					
					-- decrease luminance
					color_c := ( color_c - to_integer( to_unsigned( ( color_c * current_frame ), 32 ) srl 5 ) );
					
						-- convert cde -> rgb
						color_r_faded := ( to_unsigned( ( 298 * color_c + 409 * color_e ), 32 ) srl 16 );
						color_g_faded := ( to_unsigned( ( 298 * color_c - 100 * color_d - 208 * color_e ), 32 ) srl 16 );
						color_b_faded := ( to_unsigned( ( 298 * color_c + 516 * color_d ), 32 ) srl 16 );

					if color_r_faded( 15 ) = '1' or color_g_faded( 15 ) = '1' or color_b_faded( 15 ) = '1' then
						VGA_R <= "0000000000";
						VGA_G <= "0000000000";
						VGA_B <= "0000000000";
					else
						VGA_R <= std_logic_vector( color_r_faded( 4 downto 0 ) ) & "00000";
						VGA_G <= std_logic_vector( color_g_faded( 4 downto 0 ) ) & "00000";
						VGA_B <= std_logic_vector( color_b_faded( 4 downto 0 ) ) & "00000";
					end if;
				else
					VGA_R <= std_logic_vector( color_out( 14 downto 10 ) ) & "00000";
					VGA_G <= std_logic_vector( color_out( 9 downto 5 ) ) & "00000";
					VGA_B <= std_logic_vector( color_out( 4 downto 0 ) ) & "00000";
				end if;
      elsif vga_hblank = '0' and vga_vblank ='0' then
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "0000000000";
      else
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "0000000000";    
      end if;
    end if;

--	if EndOfField = '1' then
--		readdata(0)<='1';
--    else
--		readdata(0)<='0';
--    end if;
  end process VideoOut;

  VGA_CLK <= clk_25;
  VGA_HS <= not vga_hsync;
  VGA_VS <= not vga_vsync;
  VGA_SYNC <= '0';
  VGA_BLANK <= not (vga_hsync or vga_vsync);
end rtl;
